"""Routes package for UnClaude Web Dashboard."""

from unclaude.web.routes import chat, memory, jobs, settings, ralph, plugins, skills, mcp

__all__ = ["chat", "memory", "jobs", "settings", "ralph", "plugins", "skills", "mcp"]
